// eslint-disable-next-line func-names
(function ($, Drupal, drupalSettings, Sortable) {
  Drupal.behaviors.tagifyAutocomplete = {
    attach: function attach(context) {
      // See: https://github.com/yairEO/tagify#ajax-whitelist.
      const elements = $(once('tagify-widget', 'input.tagify-widget', context));

      // eslint-disable-next-line func-names
      elements.each(function () {
        const input = this;

        /**
         * Highlights matching letters in a given input string by wrapping them in <strong> tags.
         * @param {string} inputTerm - The input string for matching letters.
         * @param {string} searchTerm - The term to search for within the input string.
         * @return {string} The input string with matching letters wrapped in <strong> tags.
         */
        function highlightMatchingLetters(inputTerm, searchTerm) {
          // Escape special characters in the search term.
          const escapedSearchTerm = searchTerm.replace(
            /[.*+?^${}()|[\]\\]/g,
            '\\$&',
          );
          // Create a regular expression to match the search term globally and case insensitively.
          const regex = new RegExp(`(${escapedSearchTerm})`, 'gi');
          // Check if there are any matches.
          if (!escapedSearchTerm) {
            // If no matches found, return the original input string.
            return inputTerm;
          }
          // Replace matching letters with the same letters wrapped in <strong> tags.
          return inputTerm.replace(regex, '<strong>$1</strong>');
        }

        /**
         * Generates HTML markup for a tag based on the provided tagData.
         * @param {Object} tagData - Data for the tag, including value, entity_id, class, etc.
         * @return {string} - HTML markup for the generated tag.
         */
        function tagTemplate(tagData) {
          const entityIdDiv =
            parseInt(input.dataset.showEntityId, 10) && tagData.entity_id
              ? `<div id="tagify__tag-items" class="tagify__tag_with-entity-id"><div class='tagify__tag__entity-id-wrap'><span class='tagify__tag-entity-id'>${tagData.entity_id}</span></div><span class='tagify__tag-text'>${tagData.value}</span></div>`
              : `<div id="tagify__tag-items"><span class='tagify__tag-text'>${tagData.value}</span></div>`;

          return `<tag title="${tagData.value}"
            contenteditable='false'
            spellcheck='false'
            tabIndex="0"
            class="tagify__tag ${tagData.class ? tagData.class : ''}"
            ${this.getAttributes(tagData)}>
              <x id="tagify__tag-remove-button" class='tagify__tag__removeBtn' role='button' aria-label='remove tag'></x>
              ${entityIdDiv}
          </tag>`;
        }

        /**
         * Generates the HTML template for a suggestion item in the Tagify dropdown based on the provided tagData.
         * @param {Object} tagData - The data representing the suggestion item.
         * @return {string} - The HTML template for the suggestion item.
         */
        function suggestionItemTemplate(tagData) {
          return `<div ${this.getAttributes(
            tagData,
          )} class='tagify__dropdown__item ${
            tagData.class ? tagData.class : ''
          }' tabindex="0" role="option">${highlightMatchingLetters(
            tagData.value,
            this.state.inputText,
          )}</div>`;
        }

        // eslint-disable-next-line no-undef
        const tagify = new Tagify(input, {
          dropdown: {
            enabled: parseInt(input.dataset.suggestionsDropdown, 10),
            highlightFirst: true,
            fuzzySearch: !!parseInt(input.dataset.matchOperator, 10),
            maxItems: input.dataset.maxItems ?? Infinity,
            closeOnSelect: true,
          },
          templates: {
            tag: tagTemplate,
            dropdownItem: suggestionItemTemplate,
            dropdownFooter() {
              return '';
            },
          },
          whitelist: [],
          placeholder: parseInt(input.dataset.placeholder, 10),
        });
        let controller;

        // Avoid creating tag when 'Create referenced entities if they don't
        // already exist' is disallowed.
        tagify.settings.enforceWhitelist =
          !$(this).hasClass('tagify--autocreate');
        tagify.settings.skipInvalid = !!$(this).hasClass('tagify--autocreate');

        /**
         * Binds Sortable to Tagify's main element and specifies draggable items.
         */
        Sortable.create(tagify.DOM.scope, {
          // See: (https://github.com/SortableJS/Sortable#options)
          draggable: `.${tagify.settings.classNames.tag}:not(tagify__input)`,
          forceFallback: true,
          onEnd() {
            // Must update Tagify's value according to the re-ordered nodes
            // in the DOM.
            tagify.updateValueByDOMTags();
          },
        });

        /**
         * Handles autocomplete functionality for the input field using Tagify.
         * @param {string} value - The current value of the input field.
         * @param {string[]} selectedEntities - An array of selected entities.
         */
        function handleAutocomplete(value, selectedEntities) {
          tagify.whitelist = null;
          // https://developer.mozilla.org/en-US/docs/Web/API/AbortController/abort
          // eslint-disable-next-line no-unused-expressions
          controller && controller.abort();
          controller = new AbortController();
          // Show loading animation meanwhile the dropdown suggestions are hided.
          // eslint-disable-next-line no-unused-expressions
          value !== '' ? tagify.loading(true) : tagify.loading(false);
          fetch(
            `${$(input).attr('data-autocomplete-url')}?q=${encodeURIComponent(
              value,
            )}&selected=${selectedEntities}`,
            { signal: controller.signal },
          )
            .then((res) => res.json())
            // eslint-disable-next-line func-names
            .then(function (newWhitelist) {
              const newWhitelistData = [];
              // eslint-disable-next-line func-names
              newWhitelist.forEach(function (current) {
                newWhitelistData.push({
                  value: current.label,
                  entity_id: current.entity_id,
                  ...current.attributes,
                });
              });
              // Build the whitelist with the values coming from the fetch.
              if (newWhitelistData) {
                // Update whitelist Array in-place.
                tagify.whitelist = newWhitelistData;
                // Render the suggestion dropdown.
                tagify.loading(false).dropdown.show(value);
              }
            })
            .catch((error) => {
              // eslint-disable-next-line no-console
              console.error('Error fetching data:', error);
            });
        }

        // onInput event.
        // eslint-disable-next-line func-names
        const onInput = Drupal.debounce(function (e) {
          const { value } = e.detail;
          handleAutocomplete(
            value,
            tagify.value.map((item) => item.entity_id),
          );
        }, 500);

        // onEditInput event.
        // eslint-disable-next-line func-names
        const onEditInput = Drupal.debounce(function (e) {
          const value = e.detail.data.newValue;
          handleAutocomplete(
            value,
            tagify.value.map((item) => item.entity_id),
          );
        }, 500);

        // Edit input event (When user is editing the tag).
        tagify.on('edit:input', onEditInput);
        // Input event (When user is creating the tag).
        tagify.on('input', onInput);

        /**
         * Handles click events on Tagify's input, triggering autocomplete if
         * conditions are met.
         * @param {Event} e - The click event object.
         */
        function handleClickEvent(e) {
          const identifier = input.attributes.name.value;
          const containerClass = `.${identifier}`;
          const isTagifyInput = e.target.classList.contains('tagify__input');
          const isDesiredContainer = e.target.closest(containerClass);
          if (isTagifyInput && isDesiredContainer) {
            handleAutocomplete(
              '',
              tagify.value.map((item) => item.entity_id),
            );
          }
        }
        // If 'On click' dropdown suggestions is enabled.
        if (!tagify.settings.dropdown.enabled) {
          document.addEventListener('click', handleClickEvent);
        }
      });
    },
  };

  /**
   * Behaviors for tabs in the node edit form.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Attaches summary behavior for tabs in the node edit form.
   */
  Drupal.behaviors.nodeDetailsSummaries = {
    attach: function attach(context) {
      const $context = $(context);

      // eslint-disable-next-line no-shadow,func-names
      $context.find('.node-form-author').drupalSetSummary(function (context) {
        const $authorContext = $(context);
        const name = $authorContext
          .find('.field--name-uid input')
          .val()
          .split('[{"value":"')
          .pop()
          .split('",')[0];
        const date = $authorContext.find('.field--name-created input').val();

        if (name && date) {
          return Drupal.t('By @name on @date', {
            '@name': name,
            '@date': date,
          });
        }
        if (name) {
          return Drupal.t('By @name', { '@name': name });
        }
        if (date) {
          return Drupal.t('Authored on @date', { '@date': date });
        }
      });
    },
  };
})(jQuery, Drupal, drupalSettings, Sortable);
